-----------------------------------------------------------------------------------------------
-- Client Lua Script for VendorAlertMessageMover
-- Copyright (c) Jos_eu
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- VendorAlertMessageMover Module Definition
-----------------------------------------------------------------------------------------------
local VendorAlertMessageMover = {} 
local VendorAlertMessageMoverInst = nil
 
local fVendorOnDocumentReady = nil
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function VendorAlertMessageMover:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function VendorAlertMessageMover:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"Vendor",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- VendorAlertMessageMover OnLoad
-----------------------------------------------------------------------------------------------
function VendorAlertMessageMover:OnLoad()
    local vendor = Apollo.GetAddon("Vendor")

		self:LoadDefaultSettings()
		
		Apollo.RegisterSlashCommand("vamm", "OnSlashCommand", self)
		
	fVendorOnDocumentReady = vendor.OnDocumentReady
	vendor.OnDocumentReady = VendorAlertMessageMover.VendorOnDocumentReadyHook
end
-----------------------------------------------------------------------------------------------
-- VendorAlertMessageMover Functions
-----------------------------------------------------------------------------------------------

function VendorAlertMessageMover:OnSlashCommand(cmd, args)
	local n = tonumber(args)
	if n ~= nil then
		self.tSettings.Position = n
		self.RepositionAlertMessageContainer()
		Print("[VendorAlertMessageMover]: Changed alert window position to "..n)
	else
		local s = string.lower(args)
		if s == "top" then
			self.tSettings.Position = 560
			self.RepositionAlertMessageContainer()
			Print("[VendorAlertMessageMover]: Changed alert window position to top!")
		elseif s == "bottom" then
			self.tSettings.Position = 82
			self.RepositionAlertMessageContainer()
			Print("[VendorAlertMessageMover]: Changed alert window position to bottom!")
		elseif s == "below" then
			self.tSettings.Position = 15
			self.RepositionAlertMessageContainer()
			Print("[VendorAlertMessageMover]: Changed alert window position to below!")
		else
			Print("[VendorAlertMessageMover]: Could not interpret position value: "..args)
		end
	end
end

function VendorAlertMessageMover:OnRestore(eLevel, tData)
	if tData ~= nil and tData.Position ~= nil then
		local pos = tData.Position
		if type(pos) == "number" then
			self.tSettings.Position = pos
		else
			Print("VendorAlertMessageMover: Could not load position setting. Using default value.")
			self.tSettings.Position = 15
		end
	end
end

function VendorAlertMessageMover:LoadDefaultSettings()
	self.tSettings = {}
	self.tSettings.Position  = 15
end

function VendorAlertMessageMover:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
      return nil
    end
		return self.tSettings
end

function VendorAlertMessageMover.RepositionAlertMessageContainer()
	local nLeft, nTop, nRight, nBottom = VendorAlertMessageMoverInst.wndAlert:GetAnchorOffsets()
	local nTopNew = VendorAlertMessageMoverInst.tSettings.Position * -1
	if nTop ~= nTopNew then
		local nHeight = nTop - nBottom
		nTop = nTopNew
		nBottom = nTop - nHeight
		--Print(string.format("%d %d %d %d", nLeft, nTop, nRight, nBottom))
		VendorAlertMessageMoverInst.wndAlert:SetAnchorOffsets(nLeft, nTop, nRight, nBottom)
	end
end

function VendorAlertMessageMover.VendorOnDocumentReadyHook(self)
	fVendorOnDocumentReady(self)
	
	local wndAlert = self.wndVendor:FindChild("AlertMessageContainer")
	local nLeft, nTop, nRight, nBottom = wndAlert:GetAnchorOffsets()
	
	VendorAlertMessageMoverInst.nLeft = nLeft
	VendorAlertMessageMoverInst.nTop = nTop
	VendorAlertMessageMoverInst.nRight = nRight
	VendorAlertMessageMoverInst.nBottom = nBottom
	VendorAlertMessageMoverInst.wndAlert = wndAlert
	
	wndAlert:SetStyle("NoClip", true)
	wndAlert:AddEventHandler("WindowShow", "VAMM_OnAlertMessageContainerWindowShow")
	self.VAMM_OnAlertMessageContainerWindowShow = VendorAlertMessageMover.OnAlertMessageContainerWindowShow
end

function VendorAlertMessageMover.OnAlertMessageContainerWindowShow(self, wndHandler, wndControl)
	if not wndHandler then return end
	
	local strMessage = Apollo.GetString("Vendor_RestockAlert")
	local wndAlertMessageText = wndHandler:FindChild("AlertMessageText")
	
	if wndAlertMessageText:GetText() == strMessage and wndAlertMessageText:IsVisible() then
		wndHandler:SetAnchorOffsets(VendorAlertMessageMoverInst.nLeft, VendorAlertMessageMoverInst.nTop, VendorAlertMessageMoverInst.nRight, VendorAlertMessageMoverInst.nBottom)
	else
		VendorAlertMessageMoverInst.RepositionAlertMessageContainer()
	end
end

-----------------------------------------------------------------------------------------------
-- VendorAlertMessageMover Instance
-----------------------------------------------------------------------------------------------
VendorAlertMessageMoverInst = VendorAlertMessageMover:new()
VendorAlertMessageMoverInst:Init()
